/**
 * This file demonstrates the module lookup capabilities of the API. The lookup
 * allows you to create modules based on what ones are available on the network.
 */

#include <stdio.h>
#include "hebi_lookup.h"
#include "hebi_mac_address.h"
#include "hebi_util.h"

/**
 * Helper function to create mac address
 */
HebiMacAddress hebiMacAddressFromBytes(
  uint8_t a, uint8_t b, uint8_t c, uint8_t d, uint8_t e, uint8_t f)
{
  HebiMacAddress tmp;
  tmp.bytes_[0] = a;
  tmp.bytes_[1] = b;
  tmp.bytes_[2] = c;
  tmp.bytes_[3] = d;
  tmp.bytes_[4] = e;
  tmp.bytes_[5] = f;
  return tmp;
}

/**
 * Simple helper function to print out if the module is valid, and release any
 * reference to it.
 */
void checkAndReleaseModule(HebiModulePtr module)
{
  if (module == NULL)
  {
    printf("Module not found on network.\n\n");
    return;
  }

  printf("Found module.\n\n");

  // Be sure the module is released after you are done with it.
  hebiReleaseModule(module);
}

/**
 * Attempts to look for two modules on the network (hardcoded addresses below),
 * and retrieve references to them.
 */
int main()
{
  // Create the lookup object
  HebiLookupPtr lookup = hebiCreateLookup();
  if (lookup == NULL)
  {
    printf("createLookup() returned NULL.\n\n");
    hebiCleanup();
    return 1;
  }

  // Wait for the module list to populate, and print out its contents
  hebi_sleep_s(2);
  hebiPrintLookupTable(lookup);
  printf("\n");

  // Test out lookup methods; use a 4 second timeout.
  long timeout_ms = 4000;

  printf("Looking up module by MAC address.\n");
  HebiMacAddress mac = hebiMacAddressFromBytes(0x00, 0x1e, 0xc0, 0x8d, 0xe0, 0x5c);
  HebiModulePtr module = hebiGetModuleFromMac(lookup, &mac, timeout_ms);
  checkAndReleaseModule(module);

  printf("Looking up module by name.\n");
  module = hebiGetModuleFromName(lookup, "SA034", "arm", timeout_ms);
  checkAndReleaseModule(module);

  // Always delete the lookup after you are done with it.
  hebiDeleteLookup(lookup);
  hebiCleanup();

  return 0;
}
